#include "properties_filter.h"

// The code behind the macro creates a plugin and the main entries to the plugin DLL or shared object.
// The cPropertyFilter will be available through the plugins class factory.
ADTF_PLUGIN("Properties Filter Plugin",
            cPropertiesFilter);

cPropertiesFilter::cPropertiesFilter()
{
    // We register all our "visible" Property Variables in the Constructor.

    // you can set descriptions for properties, these will be shown in a tooltip when the use hovers
    // over the name of the Property in the ADTF Configuration editor.
    m_fFloatProperty.SetDescription("A property that holds floating-point values");
    // This is a hint for the ADTF Configuration Editor that defines a range of valid Property values.
    m_fFloatProperty.SetValidRange(0.0, 10.0);
    // Register the property variable.
    RegisterPropertyVariable("float_property", m_fFloatProperty);

    // To group or organise properties you can create a tree structure.
    m_nGroupedProperty1.SetDescription("Property to show grouping.");
    RegisterPropertyVariable("group/property1", m_nGroupedProperty1);
    m_nGroupedProperty2.SetDescription("Property to show another grouping.");
    RegisterPropertyVariable("group/property2", m_nGroupedProperty2);

    // This gives the user a list of Property value choices.
    m_strListProperty1.SetValueList({{"device1", "Nice Device"},
                                     {"device2", "Super Nice Device"},
                                     {"device3", "Awesome Device"}});
    m_strListProperty1.SetDescription("Property to show lists.");
    RegisterPropertyVariable("string_property", m_strListProperty1);

    // Set optional filename extension filters for the ADTF Configuration Editor FileDialogs of cFilename
    // and cFilenameList properties.
    m_oConfigFilename.SetFilenameExtensionFilter("Config files (*.xml *.json)");
    m_oConfigFilename.SetDescription("Property to show file extension filter.");
    RegisterPropertyVariable("config_filename_property", m_oConfigFilename);

    // Enums are mapped to Properties of the underlying type.
    // Here we offer the user a list to choose from.
    m_eEnumProperty.SetValueList({{eFirst, "First"},
                                  {eSecond, "Second"},
                                  {eThird, "Third"}});
    m_eEnumProperty.SetDescription("Property to show enumeration.");
    RegisterPropertyVariable("enum_property", m_eEnumProperty);

    // set basic information about the component itself and purpose
    SetDescription("This filter shows the different options using properties.");
}

void function_that_takes_an_integer(uint32_t)
{
}

tResult cPropertiesFilter::Init(tInitStage eStage)
{
    RETURN_IF_FAILED(cFilter::Init(eStage));

    // Note that the Property values are updated from the ADTF Properties File of the current Session
    // right before Init is called with eStage == StageNormal. So you can access these settings from
    // this point of your Filter life-cycle onwards.
    if (eStage == StageNormal)
    {
        // If you register a property variable in StageNormal, the associated Property will not be made
        // available via the plugin description and thus not shown in the ADTF Configuration Editor.
        // To set it, manually add it to the .adtfproperties file.
        RegisterPropertyVariable("hidden_property", m_nHiddenProperty);

        // to access the values of a Property Variable you can either just pass it to methods that
        // take parameters of the type of the property variable
        function_that_takes_an_integer(m_nGroupedProperty1);

        // or use it in calculations
        auto fProduct = m_fFloatProperty * m_nGroupedProperty2;
        LOG_INFO("product = %f", fProduct);

        // or use the pointer operator to access member methods
        LOG_INFO("string_property value is %zu characters long.",
                 m_strListProperty1->GetLength());

        // or access specific functions like
        LOG_INFO("Type of chosen config_filename_property is '%s'.", 
                 m_oConfigFilename->GetExtension().GetPtr());

        // or dereference them manually
        LOG_INFO("My properties are set to:\n"
                 " float_property = %f\n"
                 " group/property1 = %u\n"
                 " group/property2 = %u\n"
                 " string_property = %s\n"
                 " enum_property = %d\n"
                 " hidden_property = %u\n"
                 " config_filename_property = %s\n",
                 *m_fFloatProperty,
                 *m_nGroupedProperty1,
                 *m_nGroupedProperty2,
                 m_strListProperty1->GetPtr(),
                 *m_eEnumProperty,
                 *m_nHiddenProperty, 
                 m_oConfigFilename->GetPtr());
    }

    RETURN_NOERROR;
}
